# -*- coding: utf-8 -*-
#
# Copyright (c) 2018, HILSTER GmbH, Germany - https://hilster.de
# All rights reserved.
#

from __future__ import absolute_import, division, print_function, unicode_literals

import hlm.public.util
from datetime import datetime
import sys
import argparse
import webbrowser
import os.path
import hlm.licenses
import shutil
import hlm.public.activation

if sys.argv[0].endswith("__main__.py"):
    sys.argv[0] = "python -m hlm"

if sys.argv[0].endswith("hlm-script.py"):
    sys.argv[0] = "hlm"


def pretty_version(prod_lic):
    if "version" in prod_lic:
        return prod_lic['version']
    elif "min_version" in prod_lic and "max_version" in prod_lic:
        return str(prod_lic['min_version']) + " .. " + str(prod_lic['max_version'])


def pretty_expiry(prod_license):
    try:
        expiry = datetime.strptime(prod_license["expiry"], "%Y-%m-%dT%H:%M:%S")
        retval = str(expiry.date()) + (" (expired)" if prod_license['expired'] else '')
    except:
        retval = prod_license["expiry"]

    return retval


def main():
    parser = argparse.ArgumentParser(description='hlm - HILSTER License Manager (HILSTER GmbH, http://hilster.de)')
    
    subparsers = parser.add_subparsers(dest=str("command"), help='the command')
    subparsers.required = True
    
    hostid_parser = subparsers.add_parser(str('hostid'), help='query host id')
    list_parser = subparsers.add_parser(str('list'), help='list licenses available on this host')
    dongles_parser = subparsers.add_parser(str('dongles'), help='list dongles')

    # activation
    activate_parser = subparsers.add_parser(str('activate'), help="activate this node using the supplied activation keys")
    activate_parser.add_argument("-s", "--server",
                                 default="https://licenses.hilster.de",
                                 help="license server url")
    activate_parser.add_argument("--insecure",
                                 action="store_true",
                                 help="disable ssl certificate check (can be useful behind some proxy servers)")
    activate_parser.add_argument("activationkeys", nargs="+", help="activation keys")

    # renewal
    renew_parser = subparsers.add_parser(str('renew'), help="renew all activated licenses")
    renew_parser.add_argument("-s", "--server",
                                 default="https://licenses.hilster.de",
                                 help="license server url")

    docs_parser = subparsers.add_parser(str('docs'), help='open documentation')
    version_parser = subparsers.add_parser(str('version'), help='show version')

    importParser = subparsers.add_parser(str('import'), help='import a license file')
    importParser.add_argument(str('filename'), help='license file')
    importParser.add_argument(str('-f'), '--force', action="store_true", default=False, help='overwrite existing license files')
    
    license_parser = subparsers.add_parser(str('licenses'), help="show open-source licenses")

    args = parser.parse_args()

    if args.command == str("hostid"):
        hlm.public.util.print_hostid(wait=False)

    if args.command == str("list"):
        # Get search paths and licenses found for this host
        search_paths, host_licenses = hlm.public.util.get_licenses()

        # Print search paths
        print('Paths that are searched for licenses')
        print('=' * 79)
        for search_path in search_paths:
            print('- {}'.format(search_path))

        print("")

        # Print licenses found
        for host_license in host_licenses:
            # Check if there are product licenses
            if len(host_license[1]) > 0:
                print("File:     {}".format(host_license[0]))
                print("Vendor:   {}".format(host_license[1][0]['vendor']))
                print("Licensee: {}".format(host_license[1][0]['licensee']))
                print('+-{product:-<34}-+-{version:-<15}-+-{expiry:-<20}-+'.format(product="", version="", expiry=""))
                print('| {product:34} | {version:15} | {expiry:20} |'.format(
                    product="Product/Feature",
                    version="Version",
                    expiry="Expiry")
                )
                print('+-{product:-<34}-+-{version:-<15}-+-{expiry:-<20}-+'.format(product="", version="", expiry=""))

                for prod_license in host_license[1]:
                    #print(prod_license)
                    product = prod_license['product']
                    print('| {product:34} | {version:15} | {expiry:20} |'.format(
                        product=product,
                        version=pretty_version(prod_license),
                        expiry=pretty_expiry(prod_license))
                    )
                    for feature in sorted(prod_license["features"]):
                        print('| {product:34} | {version:15} | {expiry:20} |'.format(
                            product=product + "." + feature,
                            version=pretty_version(prod_license),
                            expiry=pretty_expiry(prod_license))
                        )

                print('+-{product:-<34}-+-{version:-<15}-+-{expiry:-<20}-+'.format(product="", version="", expiry=""))

            print("")

    if args.command == str("dongles"):
        print("Currently attached usb dongles:\n")
        for serial, dongleid in hlm.public.util.get_dongles_iterator():
            print(serial+":", dongleid)
        print("")

    if args.command == str("activate"):
        sys.exit(hlm.public.activation.activate_keys(args.server, args.activationkeys, insecure=args.insecure))

    if args.command == str("renew"):
        sys.exit(hlm.public.activation.renew(args.server))

    if args.command == str("docs"):
        path_to_docs = os.path.join(os.path.dirname(hlm.__file__), "doc", "html", "index.html")
        webbrowser.open("file://" + path_to_docs)
    
    if args.command == str("import"):
        filename = os.path.basename(args.filename)
        print("Importing license", args.filename)
        destination_folder = os.path.dirname(hlm.licenses.__file__)
        destination = os.path.join(destination_folder, filename)
        if os.path.exists(destination):
            if not args.force:
                raise Exception("License file already exists: %s. Use '-f' option to overwrite." % destination)
        
        print("Copying %s to %s" % (args.filename, destination))
        shutil.copy(args.filename, destination)
    
    if args.command == str("licenses"):
        print(LICENSES)
    
    if args.command == str("version"):
        print("hlm-" + hlm.__version__)


LICENSES = """
Open Source Licenses
====================

The Sodium Crypto library (libsodium-1.0.10, http://libsodium.org)
-----------------------------------------------------------

Copyright (c) 2013-2016
Frank Denis <j at pureftpd dot org>

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted, provided that the above
copyright notice and this permission notice appear in all copies.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
"""

if __name__ == "__main__":
    main()
